package com.example.flutter_bl_umpush;

import android.app.Activity;
import android.app.Notification;
import android.content.Context;
import android.os.Handler;
import android.util.SparseArray;
import android.widget.Toast;

import com.umeng.analytics.MobclickAgent;
import com.umeng.commonsdk.UMConfigure;
import com.umeng.message.IUmengCallback;
import com.umeng.message.IUmengRegisterCallback;
import com.umeng.message.MsgConstant;
import com.umeng.message.PushAgent;
import com.umeng.message.UTrack;
import com.umeng.message.UmengMessageHandler;
import com.umeng.message.UmengNotificationClickHandler;
import com.umeng.message.common.inter.ITagManager;
import com.umeng.message.entity.UMessage;
import com.umeng.message.tag.TagManager;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.os.Looper.getMainLooper;

/**
 * FlutterBlUmpushPlugin
 */
public class FlutterBlUmpushPlugin implements MethodCallHandler {

    private static Registrar registrar;
    private MethodChannel channel;
    private static String TAG = "FlutterBlUmpushPlugin";
    private Context mContext;

    private static SparseArray<Result> sCacheMap = new SparseArray<>();
    private static Result mGetRidResult;

    private static FlutterBlUmpushPlugin _me;

    private static FlutterBlUmpushPlugin me() {
        return _me;
    }

    private static Handler handler = new Handler(getMainLooper());

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        FlutterBlUmpushPlugin.registrar = registrar;
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_bl_umpush");
        channel.setMethodCallHandler(new FlutterBlUmpushPlugin());
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {

        switch (call.method) {
            case "getPlatformVersion":
                result.success("Android " + android.os.Build.VERSION.RELEASE);
            break;
            case "initWithAppkey":
                init(call, result);
                break;
            case "addAlias":
                addAlias((String) call.argument("account"), (String) call.argument("type"), result);
                break;
            case "addtags":
                addTags((List) Objects.requireNonNull(call.argument("tag")), result);
                break;
            case "delAlias":
                deleteAlias((String) call.argument("account"), (String) call.argument("type"), result);
                break;
            case "setScenarioType":
                setScenarioType(call, result);
                break;
            case "umengEnterViewWithName":
                pageStart(call, result);
                break;
            case "umengOutViewWithName":
                pageEnd(call, result);
                break;
            case "umengEventCountWithId":
                eventCounts(call, result);
                break;
            case "umengSignIn":
                umSignIn(call, result);
                break;
            case "umengSignOff":
                umSignOff(call, result);
                break;
            default:
                result.notImplemented();
        }
    }
    
    private Context getCurrentActivity() {
        
        return registrar.activity();
    }

    private void start() {
        PushAgent.getInstance(getCurrentActivity()).onAppStart();
        Logger.i(TAG, "start Success!");
    }

    // 初始化
    private void init(MethodCall call, Result result) {
        mContext = getCurrentActivity();
        initUmeng(mContext, PushAgent.getInstance(mContext), call, result);
        Logger.i(TAG, "init Push Success!");
    }

    private void initUmeng(final Context context, PushAgent mPushAgent, MethodCall call, Result result) {

        UMConfigure.setLogEnabled(true);
        UMConfigure.init(context, (String) call.argument("appKey"),
                "Umeng", UMConfigure.DEVICE_TYPE_PHONE, (String) call.argument("secret"));

        //统计SDK是否支持采集在子进程中打点的自定义事件，默认不支持
        UMConfigure.setProcessEvent(true);//支持多进程打点
        //sdk开启通知声音
        mPushAgent.setNotificationPlaySound(MsgConstant.NOTIFICATION_PLAY_SDK_ENABLE);

        //注册推送服务 每次调用register都会回调该接口
        mPushAgent.register(new IUmengRegisterCallback() {
            @Override
            public void onSuccess(String deviceToken) {
                Util.token = deviceToken;
                Logger.i(TAG, "注册推送服务 device token: " + deviceToken);
                me().channel.invokeMethod("connectionChange", Util.token != null && Util.token.length() > 0);
                if (mGetRidResult != null) {
                    mGetRidResult.success((String) Util.token);
                    mGetRidResult = null;
                }
                me().channel.invokeMethod("networkDidLogin", Util.token);
            }

            @Override
            public void onFailure(String s, String s1) {
                Util.token = null;
                me().channel.invokeMethod("connectionChange", Util.token != null && Util.token.length() > 0);
                Logger.i(TAG, "register failed: " + s + " " + s1);
            }
        });

        // 通知的回调方法（通知送达时会回调）
        UmengMessageHandler messageHandler = new UmengMessageHandler() {
            @Override
            public void dealWithNotificationMessage(Context context, UMessage msg) {
                //调用super，会展示通知，不调用super，则不展示通知。
                try {
                    if (!Logger.SHUTDOWNLOG) {
                        Logger.i(TAG, "收到推送下来的通知: " + msg.custom);
                        Logger.i(TAG, "extras: " + msg.extra.toString());
                    }
                    me().channel.invokeMethod("receivePushMsg", msg);
                } catch (Throwable e) {
                    e.printStackTrace();
                }
                super.dealWithNotificationMessage(context, msg);
            }

            /** 自定义消息的回调方法  */
            @Override
            public void dealWithCustomMessage(final Context context, final UMessage msg) {
                handler.post(new Runnable() {

                    @Override
                    public void run() {
                        // 对自定义消息的处理方式，点击或者忽略
                        try {
                            if (!Logger.SHUTDOWNLOG) {
                                Logger.i(TAG, "收到自定义消息:" + msg.custom);
                            }
                        } catch (Throwable e) {
                            e.printStackTrace();
                        }
                    }
                });
            }

            /** 自定义通知栏样式的回调方法 */
            @Override
            public Notification getNotification(Context context, UMessage msg) {
                return super.getNotification(context, msg);
            }
        };
        mPushAgent.setMessageHandler(messageHandler);

        /**
         * 自定义行为的回调处理，参考文档：高级功能-通知的展示及提醒-自定义通知打开动作
         * UmengNotificationClickHandler是在BroadcastReceiver中被调用，故
         * 如果需启动Activity，需添加Intent.FLAG_ACTIVITY_NEW_TASK
         * */
        UmengNotificationClickHandler notificationClickHandler = new UmengNotificationClickHandler() {
            @Override
            public void launchApp(Context context, UMessage msg) {
                super.launchApp(context, msg);
            }

            @Override
            public void openUrl(Context context, UMessage msg) {
                super.openUrl(context, msg);
            }

            @Override
            public void openActivity(Context context, UMessage msg) {
                super.openActivity(context, msg);
            }

            @Override
            public void dealWithCustomAction(Context context, UMessage msg) {
                Toast.makeText(context, msg.custom, Toast.LENGTH_SHORT).show();
            }
        };
        mPushAgent.setNotificationClickHandler(notificationClickHandler);

        //使用完全自定义处理
        //mPushAgent.setPushIntentServiceClass(UmengNotificationService.class);
        //小米通道
        //MiPushRegistar.register(this, XIAOMI_ID, XIAOMI_KEY);
        //华为通道
        //HuaWeiRegister.register(this);

        // true表示打开错误统计功能，false表示关闭 默认为打开
        MobclickAgent.setCatchUncaughtExceptions(true);

        // 页面采集的两种模式：AUTO和MANUAL，Android 4.0及以上版本使用AUTO,4.0以下使用MANUAL
        MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//        int results = call.argument("mode");
//        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
//            if (results == 0) {
//                // 大于等于4.4选用AUTO页面采集模式
//                MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//            }
//        } else if (results == 1) {
//            MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.MANUAL);
//        }
    }

    private void addTags(final List tags, Result result) {
        final int sequence = getSequence();
        Logger.i(TAG, "tags to be added: " + tags.toString() + " sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).getTagManager().addTags(new TagManager.TCallBack() {
            @Override
            public void onMessage(boolean isSuccess, ITagManager.Result result) {
                if (isSuccess)
                    sCacheMap.get(sequence).success(new PushResult(0, tags));
                else
                    sCacheMap.get(sequence).success(new PushResult(-1, result.jsonString));
                sCacheMap.delete(sequence);
            }
        }, getStringArrary(tags));
    }

    private void deleteTags(final List tags, Result result) {
        final int sequence = getSequence();
        Logger.i(TAG, "tags to be deleted: " + tags.toString() + " sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).getTagManager().deleteTags(new TagManager.TCallBack() {
            @Override
            public void onMessage(boolean isSuccess, ITagManager.Result result) {
                if (isSuccess)
                    sCacheMap.get(sequence).success(new PushResult(0, tags));
                else
                    sCacheMap.get(sequence).success(new PushResult(-1, result.jsonString));
                sCacheMap.delete(sequence);
            }
        }, getStringArrary(tags));
    }

    private void cleanTags(Result result) {
        result.notImplemented();
    }

    private void getAllTags(Result result) {
        final int sequence = getSequence();
        sCacheMap.put(sequence, result);
        Logger.i(TAG, "Get all tags, sequence: " + sequence);

        PushAgent.getInstance(getCurrentActivity()).getTagManager().getTags(new TagManager.TagListCallBack() {
            @Override
            public void onMessage(boolean isSuccess, List<String> list) {
                if (isSuccess)
                    sCacheMap.get(sequence).success(new PushResult(0, list));
                else
                    sCacheMap.get(sequence).success(new PushResult(-1, null));
                sCacheMap.delete(sequence);
            }
        });
    }

    private void checkTagBindState(String tag, Result result) {
        result.notImplemented();
    }

    private void addAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Logger.i(TAG, "Add alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).addAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
                sCacheMap.get(sequence).success(new PushResult(isSuccess ? 0 : -1, s));
                sCacheMap.delete(sequence);
            }
        });
    }

    private void setAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Logger.i(TAG, "Set alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).setAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
                sCacheMap.get(sequence).success(new PushResult(isSuccess ? 0 : -1, s));
                sCacheMap.delete(sequence);
            }
        });
    }

    private void deleteAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Logger.i(TAG, "Delete alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).deleteAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
                sCacheMap.get(sequence).success(new PushResult(isSuccess ? 0 : -1, s));
                sCacheMap.delete(sequence);
            }
        });
    }

    private void getAlias(Result result) {
        result.notImplemented();
    }

    private void getRegistrationID(Result result) {
        try {
            String id = Util.token;
            if (id != null) {
                result.success(id);
            } else {
                mGetRidResult = result;
            }
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    // 登陆统计
    private void umSignIn(MethodCall call, Result result) {
        MobclickAgent.onProfileSignIn((String) call.argument("userId"));
        //当用户使用第三方账号（如新浪微博）登录时，可以这样统计：
//        MobclickAgent.onProfileSignIn("WB","userID");
        // Session间隔时长,单位是毫秒，默认Session间隔时间是30秒,一般情况下不用修改此值
        MobclickAgent.setSessionContinueMillis((Long) call.argument("interval"));
        result.success(null);
    }

    // 登出
    private void umSignOff(MethodCall call, Result result) {
        MobclickAgent.onProfileSignOff();
        result.success(null);
    }

    // 打开页面
    private void pageStart(MethodCall call, Result result) {
        MobclickAgent.onPageStart((String) call.argument("name"));
        MobclickAgent.onResume(mContext);
        result.success(null);
    }

    // 关闭页面
    private void pageEnd(MethodCall call, Result result) {
        MobclickAgent.onPageEnd((String) call.argument("name"));
        MobclickAgent.onPause(mContext);
        result.success(null);
    }

    // 统计事件行为次数
    private void eventCounts(MethodCall call, Result result) {
        MobclickAgent.onEvent(mContext, (String) call.argument("eventId"));
        result.success(null);
    }

    // 场景
    private void setScenarioType(MethodCall call, Result result) {
        //EScenarioType.E_UM_NORMAL 普通统计场景，如果您在埋点过程中没有使用到
        //U-Game统计接口，请使用普通统计场景。
        //EScenarioType.E_UM_GAME 游戏场景 ，如果您在埋点过程中需要使用到U-Game
        //统计接口，则必须设置游戏场景，否则所有的U-Game统计接口不会生效。
        MobclickAgent.setScenarioType(mContext, MobclickAgent.EScenarioType.E_UM_NORMAL);
        result.success(null);
    }

    private void getConnectionState(Result result) {
        result.success(Util.token != null && !Util.token.isEmpty());
    }

    private void setLatestNotificationNumber(int number) {
        try {
            mContext = getCurrentActivity();
            PushAgent.getInstance(getCurrentActivity()).setDisplayNotificationNumber(number);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void setSilenceTime(Map map) {
        try {
            mContext = getCurrentActivity();
            List array = (List) map.get("days");
            Set<Integer> days = new HashSet<Integer>();
            for (int i = 0; i < array.size(); i++) {
                days.add((Integer) array.get(i));
            }
            int startHour = (Integer) map.get("startHour");
            int endHour = (Integer) map.get("endHour");
            PushAgent.getInstance(getCurrentActivity()).setNoDisturbMode(startHour, 0, endHour, 0);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void setResourcePackageName(Map map) {
        try {
            mContext = getCurrentActivity();
            String packageName = (String) map.get("packageName");
            PushAgent.getInstance(mContext).setResourcePackageName(packageName);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void getInfo(Result successResult) {
        Map map = new HashMap();
        String appKey = "AppKey:" + Util.getAppKey(mContext);
        map.put("myAppKey", appKey);
        String imei = "IMEI: " + Util.getImei(mContext, "");
        map.put("myImei", imei);
        String packageName = "PackageName: " + mContext.getPackageName();
        map.put("myPackageName", packageName);
        String deviceId = "DeviceId: " + Util.getDeviceId(mContext);
        map.put("myDeviceId", deviceId);
        String version = "Version: " + Util.GetVersion(mContext);
        map.put("myVersion", version);
        String toKen = "ToKen: " + Util.GetVersion(mContext);
        map.put("myToKen", toKen);
        successResult.success(map);
    }

    private void stopPush() {
        mContext = getCurrentActivity();
        PushAgent.getInstance(mContext).disable(
                new IUmengCallback() {
                    @Override
                    public void onSuccess() {
                        Logger.toast(mContext, "Stop push success");
                    }

                    @Override
                    public void onFailure(String s, String s1) {
                        Logger.i(TAG, String.format("UMPush disable Failure. %s, %s", s, s1));
                    }
                }
        );
        Logger.i(TAG, "Stop push");
    }

    private void resumePush() {
        mContext = getCurrentActivity();
        PushAgent.getInstance(mContext).enable(
                new IUmengCallback() {
                    @Override
                    public void onSuccess() {
                        Logger.toast(mContext, "Resume push success");
                    }

                    @Override
                    public void onFailure(String s, String s1) {
                        Logger.i(TAG, String.format("UMPush enable Failure. %s, %s", s, s1));
                    }
                }
        );
        Logger.i(TAG, "Resume push");
    }

    private void crashLogOFF() {
        UMConfigure.setLogEnabled(false);
    }


    private void crashLogON() {
        UMConfigure.setLogEnabled(true);
    }


    private int getSequence() {
        SimpleDateFormat sdf = new SimpleDateFormat("MMddHHmmss");
        String date = sdf.format(new Date());
        return Integer.valueOf(date);
    }

    private String[] getStringArrary(List list) {
        String[] v = new String[list.size()];
        for (int i = 0; i < list.size(); i++) {
            v[i] = list.get(i).toString();
        }
        return v;
    }

    class PushResult extends HashMap {
        PushResult(int code, Object result) {
            this.put("errorCode", code);
            this.put("data", result == null ? "" : result);
        }
    }
}
